<?php


/**
 * @file
 * This is a base connector containing the building blocks for structuring
 * Drupal data into a Juicebox gallery. This can be extended into specifc
 * plugins to handle different Drupal constructs (fields, views, etc.).
 */


abstract class JuiceboxConnector {
  
  /**
   * Base properies of structured data to manage the loading/handling of
   * Drupal info that will be processed into a gallery.
   */
  public $xmlPathArgs = array();
  public $type;
  // Library API information is stored for reference.
  public $library;
  // The raw Drupal data that will be built into a gallery is stored in mixed
  // format depending on the connector type.
  protected $data;
  // The gallery object is injected and serves as a direct interface to
  // Juicebox once Drupal data is loaded.
  public $gallery;
  // We maintain a static cache of "built" $gallery data objects, keyed by
  // gallery ID.
  static protected $builtGalleries = array();
  
  /**
   * Constructor
   * 
   * @param array $xml_path_args
   *   An indexed array of XML path arguments that describe this gallery (and
   *   make up its XML URL). This information uniquely identifies the gallery
   *   and contains all the descriptive data needed to (re)build it.
   * @param mixed $data
   *   (Optional) Loaded Drupal data that the gallery can be built from. When
   *   the embed code is being generated for a Drupal display formatter this
   *   data will typically be available already. However when building the XML
   *   this will be NULL, and we will need to be load it ourselves (based on the
   *   XML args) later.
   * @param object $gallery
   *   An empty gallery object, implementing JuiceboxGalleryInterface, that can
   *   be built into a Juicebox gallery.
   * @param array $library
   *   An associative array of the juicebox library information as structured by
   *   the libraries API.
   */
  function __construct(&$xml_path_args, $data = NULL, JuiceboxGalleryInterface $gallery, $library) {
    // Set data if we have it. If not we will need to get it later with the
    // load() method.
    if ($data) {
      $this->data = $data;
    }
    // Make sure that we have additional args to describe the Drupal data.
    if (empty($xml_path_args)) {
      throw new Exception(t('Cannot instantiate Juicebox gallery due to insufficient XML path args.'));
    }
    // Store the empty gallery object and some reference info.
    $this->gallery = $gallery;
    $this->library = $library;
    $this->type = $xml_path_args[0];
    $this->xmlPathArgs = $xml_path_args;
  }
  
  /**
   * Load the Drupal data needed to build the gallery.
   * 
   * This method should fully populate the $data propery.
   */
  abstract function load();
  
  /**
   * Check access to the Drupal data that will be used to build the gallery.
   * 
   * @return boolean
   *   Return TRUE if access is allowed for the current user and FALSE if not.
   *   Can also return NULL if access cannot be determined.
   */
  abstract function access();
  
  /**
   * Build the gallery based on loaded Drupal data specific to this connector.
   * 
   * This method should fully populate the $gallery object propery.
   */
  abstract protected function buildGallery();
  
  /**
   * Build the gallery.
   * 
   * This is essentially a wrapper for buildGallery() that ensures some common
   * pre/post-build tasks are done.
   * 
   * @param boolean $reset
   *   Whether-or-not to bypass, and reset, any statically cached gallery data.
   */
  function build($reset = FALSE) {
    // We maintain a static cache of built gallery data for efficiency. For
    // example, there may be cases where the load() method triggers a series
    // of formatter functions that also build the gallery data. Becuase build()
    // is often called right after load() (e.g. during an XML request) this can
    // prevent the build logic from running redundantly.
    if (!empty(self::$builtGalleries[$this->gallery->id]) && !$reset) {
      $this->gallery = self::$builtGalleries[$this->gallery->id];
    }
    else {
      // Build the gallery based on specific plugin logic.
      $this->buildGallery();
      // Also attempt to translate interface strings.
      if (variable_get('juicebox_translate_interface', FALSE)) {
        $base_string = variable_get('juicebox_translate_base_languagelist', '');
        if (!empty($base_string)) {
          $this->gallery->addOption('languagelist', check_plain(t($base_string)), FALSE);
        }
      }
      // Allow other modules to alter the built gallery data before it's
      // rendered. Note that the $data property is protected but we want to
      // expose it here for context.
      drupal_alter('juicebox_gallery', $this, $this->data);
      self::$builtGalleries[$this->gallery->id] = $this->gallery;
    }
  }
  
  /**
   * Helper to extract Juicebox gallery options from common Drupal display
   * settings, and add them to the gallery.
   *
   * Some common Juicebox configuration options are set via a GUI and others
   * are set as manual strings. This method fetches all of these values from
   * drupal settings data and merges them into the gallery. Note that this only
   * accounts for common settings.
   * 
   * @param array $settings
   *   An associative array of common Drupal settings for a gallery.
   */
  function setOptions($settings) {
    // Get the string options set via the GUI.
    foreach (array('jlib_galleryWidth', 'jlib_galleryHeight', 'jlib_backgroundColor', 'jlib_textColor', 'jlib_thumbFrameColor') as $name) {
      if (isset($settings[$name])) {
        $name_real = str_replace('jlib_', '', $name);
        $this->gallery->addOption(drupal_strtolower($name_real), trim(check_plain($settings[$name])));
      }
    }
    // Get the bool options set via the GUI.
    foreach (array('jlib_showOpenButton', 'jlib_showExpandButton', 'jlib_showThumbsButton', 'jlib_useThumbDots', 'jlib_useFullscreenExpand') as $name) {
      if (isset($settings[$name])) {
        $name_real = str_replace('jlib_', '', $name);
        $this->gallery->addOption(drupal_strtolower($name_real), (!empty($settings[$name]) ? 'TRUE' : 'FALSE'));
      }
    }
    // Merge-in the manually assigned options making sure they take priority
    // over any conflicting GUI options.
    $manual_options = explode("\n", $settings['manual_config']);
    foreach ($manual_options as $option) {
      $option = trim($option);
      if (!empty($option)) {
        // Each manual option has only been validated (on input) to be in the
        // form optionName="optionValue". Now we need split and sanitize the
        // values.
        $matches = array();
        preg_match('/^([A-Za-z0-9]+?)="([^"]+?)"$/u', $option, $matches);
        list($full_match, $name, $value) = $matches;
        $name = drupal_strtolower($name);
        $this->gallery->addOption(drupal_strtolower($name), check_plain($value));
      }
    }
  }
  
  /**
   * Render the embed code for a Juicebox gallery using the Drupal theme system.
   *
   * @param string $xml_path
   *   The path to the gallery's XML.
   * @param array $settings
   *   An associative array of the common Drupal settings for a gallery.
   * @return string
   *   Themed markup for the Juicebox embed code.
   */
  function renderEmbed($xml_path, $settings) {
    // Capture the current query parameters and a checksum value in the XML.
    $current_query = drupal_get_query_parameters();
    $xml_query = array_merge(array('checksum' => $this->gallery->getChecksum()), $current_query);
    $xml = array(
      'path' => $xml_path,
      'options' => array('query' => $xml_query),
    );
    // If debugging links are allowed and "jb-debug" is active in the query we
    // display debug info in place of the actual gallery.
    $settings['show_debug_links'] = variable_get('juicebox_show_debug_links', FALSE);
    if ($settings['show_debug_links'] && array_key_exists('jb-debug', $current_query)) {
      drupal_add_css(drupal_get_path('module', 'juicebox') . '/css/placeholder_styles.css');
      return theme('juicebox_debug_markup', array('gallery' => $this->gallery, 'xml' => $xml, 'settings' => $settings));
    }
    else {
      // Load the juicebox javascript library and gallery-specific javascript.
      libraries_load('juicebox');
      $xml_url = url($xml['path'], $xml['options']);
      drupal_add_js($this->gallery->getJavascript($xml_url), array('type' => 'inline', 'scope' => variable_get('juicebox_js_scope', 'header')));
      // Theme the embed code.
      return theme('juicebox_embed_markup', array('gallery' => $this->gallery, 'settings' => $settings));
    }
  }
  
  /**
   * Utility to extract image source data in an array structure that can be
   * added to a Juicebox gallery object.
   *
   * @param array $image_item
   *   An associative array of file field item data for the main image.
   * @param string $image_style
   *   The Drupal image style to apply to the main image.
   * @param array $thumb_item
   *   An associative array of file field item data for the thumbnail
   *   image.
   * @param string $thumb_style
   *   The Drupal image style to apply to the thumbnail image.
   * @param array $settings
   *   Common Drupal settings for a Juicebox gallery.
   * @return array
   *   An associative array of image source URLs that's ready to be added
   *   to a Juicebox gallery.
   */
  protected function getImageSrcData($image_item, $image_style, $thumb_item, $thumb_style, $settings) {
    // Make sure that we have proper file field items to work with.
    if (!isset($image_item['fid']) || !isset($thumb_item['fid'])) {
      throw new Exception(t('Invalid image or thumbnail item detected while extracting Juicebox gallery source information.'));
    }
    $check_incompatible = (!empty($settings['incompatible_file_action']));
    $src_data = array();
    // Style the main item.
    $image_item = $this->styleImage($image_item, $image_style, $check_incompatible);
    // Set image data.
    $src_data['image_url'] = $image_item['juicebox_styled_src'];
    // Set thumb data.
    $src_data['thumb_url'] = '';
    if (!$image_item['juicebox_compatible'] && $image_item['fid'] == $thumb_item['fid']) {
      $src_data['thumb_url'] = $image_item['juicebox_styled_src'];
    }
    else {
      $thumb_item = $this->styleImage($thumb_item, $thumb_style, $check_incompatible);
      $src_data['thumb_url'] = $thumb_item['juicebox_styled_src'];
    }
    // Check if the linkURL should be customized based on settings.
    $src_data['link_url'] = $image_item['juicebox_unstyled_src'];   
    if ($image_item['juicebox_compatible'] && !empty($settings['linkurl_source']) && $settings['linkurl_source'] == 'image_styled') {
      $src_data['link_url'] = $image_item['juicebox_styled_src'];
    }
    // Set the link target directly from the gallery settings.
    $src_data['link_target'] = $settings['linkurl_target'];
    // Also set a flag for incompatible files.
    $src_data['juicebox_compatible'] = $image_item['juicebox_compatible'];
    return $src_data;
  }
  
  /**
   * Utility to style an individual image/file field item for use in a Juicebox
   * gallery.
   * 
   * This method can detect if the passed item is incompatible with Juicebox.
   * If so it styles the output as a mimetype image icon representing the file
   * type. Otherwise the item is styled normally with the passed image style.
   *
   * @param array $item
   *   An associative array of file field item data.
   * @param string $style
   *   The Drupal image style to apply to the item.
   * @param boolean $check_compatible
   *   Whether-or-not to detect if the item is compatible with Juicebox, and if
   *   so, substitute a mimetype icon for its output.
   * @return array
   *   The file item with style data appended to the array.
   */
  protected function styleImage($item, $style, $check_compatible = TRUE) {
    $item['juicebox_compatible'] = TRUE;
    $item['juicebox_styled_src'] = '';
    // Set the normal, unstyled, url for reference.
    $item['juicebox_unstyled_src'] = file_create_url($item['uri']);
    // Check compatibility if configured and if the library info contains
    // mimetype compatibitly information.
    if ($check_compatible && !empty($this->library['compatible_mimetypes']) && !in_array($item['filemime'], $this->library['compatible_mimetypes'])) {
      // If the item is not compatible, find the substitute mimetype icon.
      $item['juicebox_compatible'] = FALSE;
      $icon_dir = drupal_get_path('module', 'juicebox') . '/images/mimetypes';
      $icon_path = file_icon_path((object) $item, $icon_dir);
      if ($icon_path) {
        $item['juicebox_styled_src'] = url($icon_path);
      }
      else {
        $item['juicebox_styled_src'] = url($icon_dir . '/image-x-generic.png');
      }
    }
    // If the item is compatible, style it.
    elseif (!empty($style)) {
      $item['juicebox_styled_src'] = image_style_url($style, $item['uri']);
    }
    else {
      $item['juicebox_styled_src'] = $item['juicebox_unstyled_src'];
    }
    return $item;
  }
  
  /**
   * Get the "base" values of common Drupal settings used to describe a gallery.
   * This is used for the management of default configuration values.
   * 
   * Note that this may be called when building Drupal configuration forms that
   * are managed outside of this object.
   *
   * @return array
   *   An associative array of base/default configuration values. 
   */
  static function confBaseOptions() {
    return array(
      'jlib_galleryWidth' => '100%',
      'jlib_galleryHeight' => '100%',
      'jlib_backgroundColor' => '#222222',
      'jlib_textColor' => 'rgba(255,255,255,1)',
      'jlib_thumbFrameColor' => 'rgba(255,255,255,.5)',
      'jlib_showOpenButton' => 1,
      'jlib_showExpandButton' => 1,
      'jlib_showThumbsButton' => 1,
      'jlib_useThumbDots' => 0,
      'jlib_useFullscreenExpand' => 0,
      'manual_config' => '',
      'custom_parent_classes' => '',
      'apply_markup_filter' => 1,
      'linkurl_source' => '',
      'linkurl_target' => '_blank',
      'incompatible_file_action' => 'show_icon_and_link',
      'show_debug_links' => 0,
    );
  }
  
  /**
   * Get common elements for Juicebox configuration forms.
   *
   * Several Juicebox gallery types can share common options and structures.
   * These are merged into the appropriate forms via a call to this method. Note
   * that this may be called when building Drupal configuration forms that are
   * managed outside of this object.
   *
   * @param array $form
   *   The Drupal form array that common elements should be added to.
   * @param array $settings
   *   An associative array containing all the current settings for a Juicebox
   *   gallery (used to set default values).
   * @return array
   *   The common form elements merged within a form array.
   */
  static function confBaseForm($form, $settings) {
    // Get locally installed library details.
    $library = juicebox_library_detect();
    $disallowed_conf = array();
    if (!empty($library)) {
      // If we don't have a known version of the Juicebox library, just show a
      // generic warning.
      if (empty($library['version'])) {
        $notification_top = t('<strong>Notice:</strong> Your Juicebox Library version could not be detected. Some options below may not function correctly.');
      }
      // If this version does not support some LITE optins, show a message.
      elseif (!empty($library['disallowed_conf'])) {
        $disallowed_conf = $library['disallowed_conf'];
        $notification_top = t('<strong>Notice:</strong> You are currently using Juicebox library version <strong>@version</strong> which is not compatible with some of the options listed below. These options will appear disabled until you upgrade to the most recent Juicebox library version.', array('@version' => $library['version']));
        $notification_label = t('&nbsp;(not available in @version)', array('@version' => $library['version']));
      }
    }
    // If the library itself is not installed, display formal error message.
    else {
      $notification_top = t('The Juicebox Javascript library does not appear to be installed. Please download and install the most recent version of the Juicebox library.');
      drupal_set_message($notification_top, 'error');
    }
    $form['juicebox_config'] = array(
      '#type' => 'fieldset',
      '#title' => t('Juicebox Library - Lite Config'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => !empty($notification_top) ? '<p>' . $notification_top . '</p>' : '',
      '#weight' => 10,
    );
    $form['jlib_galleryWidth'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'textfield',
      '#title' => t('Gallery Width'),
      '#description' => t('Set the gallery width in a standard numeric format (such as 100% or 300px).'),
      '#element_validate' => array('juicebox_element_validate_dimension'),
    );
    $form['jlib_galleryHeight'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'textfield',
      '#title' => t('Gallery Height'),
      '#description' => t('Set the gallery height in a standard numeric format (such as 100% or 300px).'),
      '#element_validate' => array('juicebox_element_validate_dimension'),
    );
    $form['jlib_backgroundColor'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'textfield',
      '#title' => t('Background Color'),
      '#description' => t('Set the gallery background color as a CSS3 color value (such as rgba(10,50,100,0.7) or #FF00FF).'),
    );
    $form['jlib_textColor'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'textfield',
      '#title' => t('Text Color'),
      '#description' => t('Set the color of all gallery text as a CSS3 color value (such as rgba(255,255,255,1) or #FF00FF).'),
    );
    $form['jlib_thumbFrameColor'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'textfield',
      '#title' => t('Thumbnail Frame Color'),
      '#description' => t('Set the color of the thumbnail frame as a CSS3 color value (such as rgba(255,255,255,.5) or #FF00FF).'),
    );
    $form['jlib_showOpenButton'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'checkbox',
      '#title' => t('Show Open Image Button'),
      '#description' => t('Whether to show the "Open Image" button. This will link to the full size version of the image within a new tab to facilitate downloading.'),
    );
    $form['jlib_showExpandButton'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'checkbox',
      '#title' => t('Show Expand Button'),
      '#description' => t('Whether to show the "Expand" button. Clicking this button expands the gallery to fill the browser window.'),
    );
    $form['jlib_useFullscreenExpand'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'checkbox',
      '#title' => t('Use Fullscreen Expand'),
      '#description' => t('Whether to trigger fullscreen mode when clicking the expand button (for supported browsers).'),
    );
    $form['jlib_showThumbsButton'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'checkbox',
      '#title' => t('Show Thumbs Button'),
      '#description' => t('Whether to show the "Toggle Thumbnails" button.'),
    );
    $form['jlib_useThumbDots'] = array(
      '#jb_fieldset' => 'juicebox_config',
      '#type' => 'checkbox',
      '#title' => t('Show Thumbs Dots'),
      '#description' => t('Whether to replace the thumbnail images with small dots.'),
      '#default_value' => $settings['jlib_useThumbDots'],
    );
    $form['juicebox_manual_config'] = array(
      '#type' => 'fieldset',
      '#title' => t('Juicebox Library - Pro / Manual Config'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => '<p>' . t('Specify any additional Juicebox library configuration options (such as "Pro" options) here.<br/>Options set here always take precedence over those set in the "Lite" options above if there is a conflict.') . '</p>',
      '#weight' => 20,
    );
    $form['manual_config'] = array(
      '#jb_fieldset' => 'juicebox_manual_config',
      '#type' => 'textarea',
      '#title' => t('Pro / Manual Configuraton Options'),
      '#description' => t('Add one option per line in the format <strong>optionName="optionValue"</strong><br/>See also: http://www.juicebox.net/support/config_options'),
      '#element_validate' => array('juicebox_element_validate_config'),
    );
    $form['advanced'] = array(
      '#type' => 'fieldset',
      '#title' => t('Juicebox - Advanced Options'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#weight' => 30,
    );
    $form['incompatible_file_action'] = array(
      '#jb_fieldset' => 'advanced',
      '#type' => 'select',
      '#title' => t('Incompatible File Type Handling'),
      '#options' => array('skip' => 'Bypass incompatible files', 'show_icon' => 'Show mimetype icon placehoder', 'show_icon_and_link' => 'Show mimetype icon placholder and link to file'),
      '#empty_option' => t('Do nothing'),
      '#description' => t('Specify any special handling that should be applied to files that Juicebox cannot display (non-images).'),
    );
    $form['linkurl_source'] = array(
      '#jb_fieldset' => 'advanced',
      '#type' => 'select',
      '#title' => t("LinkURL Source"),
      '#description' => t('The linkURL is an image-specific path for accessing each image outside the gallery. This is used by features such as the "Open Image Button".'),
      '#options' => array('image_styled' => 'Main Image - Styled (use this gallery\'s main image style setting)'),
      '#empty_option' => t('Main Image - Unstyled (original image)'),
    );
    $form['linkurl_target'] = array(
      '#jb_fieldset' => 'advanced',
      '#type' => 'select',
      '#title' => t('LinkURL Target'),
      '#options' => array('_blank' => '_blank', '_self' => '_self', '_parent' => '_parent', '_top' => '_top'),
      '#description' => t('Specify a target for any links that make user of the image linkURL.'),
    );
    $form['custom_parent_classes'] = array(
      '#jb_fieldset' => 'advanced',
      '#type' => 'textfield',
      '#title' => t('Custom Classes for Parent Container'),
      '#description' => t('Define any custom classes that should be added to the parent container within the Juicebox embed markup.<br/>This can be handy if you want to apply more advanced styling or dimensioning rules to this gallery via CSS. Enter as space-separated values.'),
    );
    // Set values that are directly related to each key.
    foreach($form as $conf_key => &$conf_value) {
      if (!empty($conf_value['#type']) && $conf_value['#type'] != 'fieldset') {
        $conf_value['#default_value'] = $settings[$conf_key];
        if (in_array($conf_key, $disallowed_conf)) {
          $conf_value['#title'] .= $notification_label;
          $conf_value['#disabled'] = TRUE;
        }
      }
    }
    // Add a pre render callback that will ensure that the items are nested
    // correctly into fieldsets just before display.
    $form['#pre_render'] = array('juicebox_form_pre_render_fieldsets');
    return $form;
  }

}
